package cn.com.qmth.stmms.ms.marking.api;

import cn.com.qmth.stmms.ms.commons.config.ScoreConfig;
import cn.com.qmth.stmms.ms.commons.utils.SqlUtil;
import cn.com.qmth.stmms.ms.commons.web.PageableDTO;
import cn.com.qmth.stmms.ms.core.domain.ExamQuestion;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.domain.enums.TrialEnum;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTask;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.repository.*;
import cn.com.qmth.stmms.ms.marking.assembler.MarkTaskAssembler;
import cn.com.qmth.stmms.ms.marking.dto.MarkTaskDTO;
import cn.com.qmth.stmms.ms.marking.service.MarkingService;
import com.google.gson.Gson;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.persistence.criteria.Predicate;
import java.util.*;

/**
 * 评卷任务数据api
 * Created by zhengmin on 2016/10/12.
 */
@RestController
@RequestMapping("api/marktasks")
public class MarkTaskApi {
    private static Logger LOGGER = LoggerFactory.getLogger(MarkTaskApi.class);

    @Autowired
    private MarkTaskRepo markTaskRepo;

    @Autowired
    private MarkingService markingService;

    @Autowired
    private MarkTaskAssembler markTaskAssembler;

    @Autowired
    ExamQuestionRepo examQuestionRepo;

    @Autowired
    PaperRepo paperRepo;

    @Autowired
    MarkUserRepo markUserRepo;

    @Autowired
    ScoreConfig scoreConfig;

    @Autowired
    LevelRepo levelRepo;

    @Autowired
    SqlUtil sqlUtil;

    /**
     * 评卷员的评卷任务
     *
     * @param markerId   评卷员用户id
     * @param stage      评卷阶段
     * @param level      档位
     * @param sn         密号
     * @param questionId 试题id
     * @return
     */
    @RequestMapping(method = RequestMethod.GET)
    public PageableDTO list(@RequestParam Long markerId,
                            @RequestParam MarkStage stage,
                            @RequestParam(required = false) Boolean isSample,
                            @RequestParam(required = false) String level,
                            @RequestParam(required = false) String sn,
                            @RequestParam Long questionId,
                            Pageable pageable) throws Exception {
        List<MarkTaskDTO> markTaskDTOs = new ArrayList<>();
        Specification<MarkTask> specification = (root, query, builder) -> {
            List<Predicate> predicates = new ArrayList<>();
            predicates.add(builder.equal(root.get("questionId"), questionId));
            predicates.add(builder.equal(root.get("markerId"), markerId));
            predicates.add(builder.equal(root.get("stage"), stage));
            if (level == null) {
                predicates.add(builder.isNull(root.get("result")));
            } else if (stage == MarkStage.LEVEL) {
                //查询
                Long batchNo = paperRepo.findByQuestionId(questionId);
                predicates.add(builder.equal(root.get("result"), level));
                if (!Objects.isNull(batchNo)) {
                    predicates.add(builder.equal(root.get("paper").get("batchNo"), batchNo));
                }
            } else if (stage == MarkStage.SCORE) {
                predicates.add(builder.isNotNull(root.get("result")));
            }
            if (isSample != null) {
                predicates.add(builder.equal(root.get("paper").get("isSample"), isSample));
            }
            predicates.add(builder.equal(root.get("paper").get("isMissing"), false));
            predicates.add(builder.equal(root.get("paper").get("active"), true));
//            predicates.add(builder.equal(root.get("isActive"), false));
            return builder.and(predicates.toArray(new Predicate[predicates.size()]));
        };

        Page<MarkTask> markTasks = markTaskRepo.findAll(specification, pageable);
        if (Objects.isNull(markTasks) || markTasks.getContent().size() == 0) {
            if (Objects.nonNull(isSample) && isSample) {
                ExamQuestion examQuestion = examQuestionRepo.findOne(questionId);
                if (examQuestion.getTest() == TrialEnum.DEFAULT.getId()) {
                    String sql = "select p.id, p.work_id as workId, p.secret_number as secretNumber, p.idx, p.student_name as studentName, p.exam_number as examNumber, p.subject, p.question_id as questionId, p.area_code as areaCode, p.question_name as questionName, p.level, p.redo_level as redoLevel, p.score, p.inspect_score as inspectScore, p.inspect_level as inspectLevel, p.inspector, p.is_arbitrated as isArbitrated, p.is_rejected as isRejected, p.mark_by_leader as markByLeader, p.marked_logic as markedLogic, p.updated_on as updatedOn, p.is_manual as isManual, p.uploaded_on as uploadedOn, p.uploaded_count as uploadedCount, p.is_tagged as isTagged, p.is_sample as isSample, p.inspect_range as inspectRange, p.source_name as sourceName, p.sheet_md5 as sheetMD5, p.slice_md5 as sliceMD5, p.is_missing as isMissing, p.random_seq as randomSeq, p.exam_room as examRoom, p.is_test as test, p.is_active as active, p.batch_no as batchNo from paper p where p.`level` = ? and p.`is_sample` = ? and p.`is_missing` = ? and p.`subject` = ? and (p.`is_test` = ? or p.`is_test` = ?) and p.`area_code` = ? and p.`work_id` = ? order by p.`updated_on` desc";
                    sql = sqlUtil.sqlPrint(sql, "'" + level + "'", isSample, false, "'" + examQuestion.getSubject() + "'", TrialEnum.DEFAULT.getId(), TrialEnum.START_FORMAL.getId(), "'" + examQuestion.getAreaCode() + "'", examQuestion.getWorkId());
                    List paperList = sqlUtil.execSqlForMap(sql);
                    if (Objects.nonNull(paperList) && paperList.size() > 0) {
                        MarkUser markUser = markUserRepo.findOne(markerId);
                        Gson gson = new Gson();
                        for (int i = 0; i < paperList.size(); i++) {
                            Map map = (Map) paperList.get(i);
                            Paper p = gson.fromJson(gson.toJson(map), Paper.class);
                            MarkTask markTask = new MarkTask(markUser, p, stage, 1L);
                            markTaskDTOs.add(markTaskAssembler.toDTO(markTask));
                        }
                    }
                }
            }
        }
        markTasks.getContent().forEach(m -> {
            markTaskDTOs.add(markTaskAssembler.toDTO(m));
        });
        if (stage == MarkStage.SCORE && Objects.isNull(level)) {
            if (Objects.nonNull(scoreConfig) && Objects.nonNull(scoreConfig.getTaskSort()) && Objects.equals("paper", scoreConfig.getTaskSort())) {
                Collections.sort(markTaskDTOs, Comparator.comparing(MarkTaskDTO::getPaperId));
            } else {
                Collections.sort(markTaskDTOs, Comparator.comparing(MarkTaskDTO::getRandomSeqNew));
            }
        } else if (stage == MarkStage.LEVEL && Objects.isNull(level)) {
            Collections.sort(markTaskDTOs, Comparator.comparing(MarkTaskDTO::getRandomSeqNew));
        }
        return new PageableDTO(markTaskDTOs, markTasks.getTotalElements(), markTasks.getTotalPages(), pageable.getPageNumber());
    }

    /**
     * 提交评卷任务
     *
     * @param markTask 评卷你任务id
     * @param body     评卷内容
     * @return
     */
    @RequestMapping(value = "{markTask}", method = RequestMethod.PATCH)
    public ResponseEntity marking(@PathVariable MarkTask markTask, @RequestBody HashMap<String, String> body) {
        MarkStage stage = MarkStage.valueOf(body.get("stage"));
        String result = body.get("result");
        boolean oldRejected = markTask.isRejected();
        if (stage == null || result == null) {
            return new ResponseEntity(HttpStatus.BAD_REQUEST);
        }
        switch (stage) {
            case LEVEL:
                markingService.levelMark(markTask, result);
                break;
            case SCORE:
                Integer score = Integer.parseInt(result);
                markingService.scoring(markTask, score);
                break;
        }
        return new ResponseEntity(markTaskAssembler.toDTO(markTask, oldRejected), HttpStatus.OK);
    }

    /**
     * 跳过评卷任务
     *
     * @param markTaskId 评卷你任务id
     * @return
     */
    @RequestMapping(value = "{markTaskId}/skip", method = RequestMethod.POST)
    public ResponseEntity skip(@PathVariable Long markTaskId) {
        MarkTask markTask = markingService.skip(markTaskId);
        return new ResponseEntity(markTaskAssembler.toDTO(markTask), HttpStatus.OK);
    }

}
